#include "CarParking.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <ranges>
#include <algorithm>
#include <iomanip>

parking::parking(const char* t_owner_name, int t_car_number, double t_tariff, int t_daystart, int t_dayend, int t_timestart, int t_timeend)
	:car_number(t_car_number)
	, tariff(t_tariff)
	, day_start(t_daystart)
	, day_end(t_dayend)
	, time_start(t_timestart)
	, time_end(t_timeend)
{
	strcpy_s(owner_name, 100, t_owner_name);
}

std::istream& operator >> (std::istream& is, parking& t_parking)
{
	is.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
	std::cout << "Owner's name: ";
	is.getline(t_parking.owner_name, 100);
	std::cout << "Car Number: ";
	is >> t_parking.car_number;
	std::cout << "Tariff: ";
	is >> t_parking.tariff;
	std::cout << "Day Start Parking: ";
	is >> t_parking.day_start;
	std::cout << "Day End Parking: ";
	is >> t_parking.day_end;
	std::cout << "Time Start Parking: ";
	is >> t_parking.time_start;
	std::cout << "Time End Parking: ";
	is	>> t_parking.time_end;
	return is;
}

std::ostream& operator << (std::ostream& os, const parking& t_parking)
{
	os << t_parking.owner_name << std::endl << t_parking.car_number <<std::endl
		<< t_parking.tariff<<std::endl << t_parking.day_start<<std::endl
		<< t_parking.day_end <<std::endl<< t_parking.time_start<<std::endl << t_parking.time_end<<std::endl;
	return os;
}


Record::Record(const std::string& r_name)
	:name_record(r_name) 
{}

void Record::create_record() {
	std::ofstream ofstr(name_record, std::ios::binary);
	int option;
	parking p;
	std::cout << "Enter the information of new parking slot (Enter the first line to start the input):" << std::endl;
	std::cout << "Owner's name - Car Number - Tariff per hour - Day Start - Day End - Time Start - Time End" << std::endl;
	do {
		std::cin >> p;
		ofstr.write(reinterpret_cast<char*>(&p), sizeof(p));
		std::cout << "0 - END" << std::endl << "1 - CONTINUE" << std::endl << "Enter option: ";
		std::cin >> option;
		if (option < 0 && option > 1) {
			std::cout << "Not request like that" << std::endl;
			break;
		}
	} while (option == 1);
}

void Record::get_data()
{
	std::ifstream ifstr(name_record, std::ios::binary);
	parking p;
	int idx = 0;
	while (ifstr.read(reinterpret_cast<char*>(&p), sizeof(p))) {
		record_map.insert(std::make_pair(++idx, p));
	}
}

void Record::add_to_record(const parking& t_parking) {
	std::ofstream ofstr(name_record, std::ios::binary | std::ios::app);
	ofstr.write(reinterpret_cast<const char*>(&t_parking), sizeof(parking));
}

void Record::add_to_record_list(const std::initializer_list<parking>& p_list)
{
	std::ofstream ofstr(name_record, std::ios::binary | std::ios::app);
	for (const auto& ele : p_list)
		ofstr.write(reinterpret_cast<const char*>(&ele), sizeof(parking));
}

void Record::delete_from_record(int idx)
{
	get_data();
	record_map.erase(idx);
	std::ofstream fout(name_record + "_updated", std::ios::binary);
	for (const auto& ele : record_map)
		fout.write(reinterpret_cast<const char*>(&ele.second), sizeof(parking));
}

void Record::replace_in_record(int idx, const parking& p) {
	get_data();
	std::ofstream fout(name_record + "_updated", std::ios::binary);
	record_map[idx] = p;
	for (const auto& ele : record_map)
		fout.write(reinterpret_cast<const char*>(&ele.second), sizeof(p));
}

void Record::transform_into_text() {
	get_data();
	std::ofstream fout(name_record + ".txt");
	fout << "--------------------------------------------------------- Parking Slot ----------------------------------------------------- " << std::endl;
	fout << "| No |  Owner's Name  |   Car Number   |     Tariff     |    Day Start   |     Day End    |   Time Start   |    Time End    |" << std::endl;
	size_t count = 0;
	for (const auto& ele : record_map) {
		count++;
		fout << "|" << std::setw(4) << count << "|"
			<< std::setw(16) << ele.second.owner_name << "|"
			<< std::setw(16) << ele.second.car_number<< "|"
			<< std::setw(16) << ele.second.tariff << "|"
			<< std::setw(16) << ele.second.day_start << "|"
			<< std::setw(16) << ele.second.day_end << "|"
			<< std::setw(16) << ele.second.time_start<< "|"
			<< std::setw(16) << ele.second.time_end<< "|"
			<< std::endl;
	}
}

void Record::search_by_name(const std::string& t_name)
{
	get_data();
	std::ofstream fout(t_name, std::ios::binary);
	std::vector<parking> res;
	std::ranges::copy(
		record_map
		| std::ranges::views::transform([](const std::pair<int, parking>& p) {return p.second; })
		| std::ranges::views::filter([&t_name](const parking& elem) { return elem.owner_name == t_name; })
		, std::back_inserter(res));

	for (const auto& e : res)
		fout.write(reinterpret_cast<const char*>(&e), sizeof(parking));
}

void Record::search_by_car_number(int t_num) {
	get_data();
	std::ofstream ofstr(std::to_string(t_num), std::ios::binary);
	std::vector<parking> res;
	std::ranges::copy(
		record_map
		| std::ranges::views::transform([](const std::pair<int, parking>& p) {return p.second; })
		| std::ranges::views::filter([&t_num](const parking& elem) { return elem.car_number == t_num; })
		, std::back_inserter(res));
	for (const auto& e : res)
		ofstr.write(reinterpret_cast<const char*>(&e), sizeof(parking));
}

void Record::search_by_date(int t_date) {
	get_data();
	std::ofstream ofstr(std::to_string(t_date), std::ios::binary);
	std::vector<parking> res;
	std::ranges::copy(
		record_map
		| std::ranges::views::transform([](const std::pair<int, parking>& p) {return p.second; })
		| std::ranges::views::filter([&t_date](const parking& ele) { return ele.day_start == t_date || ele.day_end == t_date; })
		, std::back_inserter(res));
	for (const auto& e : res)
		ofstr.write(reinterpret_cast<const char*>(&e), sizeof(parking));
}

void Record::search_by_time(int t_time) {
	get_data();
	std::ofstream ofstr(std::to_string(t_time), std::ios::binary);
	std::vector<parking> res;
	std::ranges::copy(
		record_map
		| std::ranges::views::transform([](const std::pair<int, parking>& p) {return p.second; })
		| std::ranges::views::filter([&t_time](const parking& ele) { return ele.time_start == t_time || ele.time_end == t_time; })
		, std::back_inserter(res));
	for (const auto& e : res)
		ofstr.write(reinterpret_cast<const char*>(&e), sizeof(parking));
}







